@@ -1,5 +1,5 @@
 //
-// Copyleft 2012 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2012-06-05 GONG Chen <chen.sst@gmail.com>
@@ -17,9 +17,10 @@ namespace rime {
 
 class ChordComposer : public Processor {
  public:
-  ChordComposer(Engine *engine);
+  ChordComposer(const Ticket& ticket);
+  ~ChordComposer();
 
-  virtual Result ProcessKeyEvent(const KeyEvent &key_event);
+  virtual ProcessResult ProcessKeyEvent(const KeyEvent& key_event);
 
  protected:
   std::string SerializeChord();
@@ -27,6 +28,8 @@ class ChordComposer : public Processor {
   void FinishChord();
   void ClearChord();
   bool DeleteLastSyllable();
+  void OnContextUpdate(Context* ctx);
+  void OnUnhandledKey(Context* ctx, const KeyEvent& key);
 
   std::string alphabet_;
   std::string delimiter_;
@@ -36,7 +39,11 @@ class ChordComposer : public Processor {
 
   std::set<char> pressed_;
   std::set<char> chord_;
-  bool pass_thru_;
+  bool pass_thru_ = false;
+  bool composing_ = false;
+  std::string sequence_;
+  connection update_connection_;
+  connection unhandled_key_connection_;
 };
 
 }  // namespace rime
