@@ -1,5 +1,5 @@
 //
-// Copyleft 2011 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2011-06-20 GONG Chen <chen.sst@gmail.com>
@@ -13,10 +13,10 @@ namespace rime {
 
 class EchoTranslator : public Translator {
  public:
-  EchoTranslator(const TranslatorTicket& ticket);
+  EchoTranslator(const Ticket& ticket);
 
-  virtual shared_ptr<Translation> Query(const std::string &input,
-                                        const Segment &segment,
+  virtual shared_ptr<Translation> Query(const std::string& input,
+                                        const Segment& segment,
                                         std::string* prompt);
 };
 
