@@ -1,5 +1,5 @@
 //
-// Copyleft 2013 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2013-01-02 GONG Chen <chen.sst@gmail.com>
@@ -9,7 +9,6 @@
 
 #include <string>
 #include <vector>
-#include <boost/signals/connection.hpp>
 #include <rime/common.h>
 #include <rime/dict/vocabulary.h>
 
@@ -39,7 +38,7 @@ class Language {
 
 class Memory {
  public:
-  Memory(Engine* engine, const std::string& name_space);
+  Memory(const Ticket& ticket);
   virtual ~Memory();
 
   virtual bool Memorize(const CommitEntry& commit_entry) = 0;
@@ -54,13 +53,13 @@ class Memory {
   void OnDeleteEntry(Context* ctx);
   void OnUnhandledKey(Context* ctx, const KeyEvent& key);
 
-  scoped_ptr<Dictionary> dict_;
-  scoped_ptr<UserDictionary> user_dict_;
+  unique_ptr<Dictionary> dict_;
+  unique_ptr<UserDictionary> user_dict_;
 
  private:
-  boost::signals::connection commit_connection_;
-  boost::signals::connection delete_connection_;
-  boost::signals::connection unhandled_key_connection_;
+  connection commit_connection_;
+  connection delete_connection_;
+  connection unhandled_key_connection_;
   Language language_;
 };
 
