@@ -1,5 +1,5 @@
 //
-// Copyleft 2011 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2012-01-01 GONG Chen <chen.sst@gmail.com>
@@ -20,28 +20,28 @@ class Segmentation;
 
 struct RecognizerMatch {
   std::string tag;
-  size_t start, end;
+  size_t start = 0, end = 0;
+
+  RecognizerMatch() = default;
+  RecognizerMatch(const std::string& a_tag, size_t a_start, size_t an_end)
+      : tag(a_tag), start(a_start), end(an_end) {}
 
-  RecognizerMatch() : tag(), start(0), end(0) {}
-  RecognizerMatch(const std::string &_tag, size_t _start, size_t _end)
-      : tag(_tag), start(_start), end(_end) {}
-  
   bool found() const { return start < end; }
 };
 
 class RecognizerPatterns : public std::map<std::string, boost::regex> {
  public:
-  void LoadConfig(Config *config);
-  RecognizerMatch GetMatch(const std::string &input,
-                           Segmentation *segmentation) const;
+  void LoadConfig(Config* config);
+  RecognizerMatch GetMatch(const std::string& input,
+                           Segmentation* segmentation) const;
 };
 
 class Recognizer : public Processor {
  public:
-  Recognizer(Engine *engine);
-  
-  virtual Result ProcessKeyEvent(const KeyEvent &key_event);
-  
+  Recognizer(const Ticket& ticket);
+
+  virtual ProcessResult ProcessKeyEvent(const KeyEvent& key_event);
+
  protected:
   RecognizerPatterns patterns_;
 };
