@@ -1,5 +1,5 @@
 //
-// Copyleft 2011 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2011-12-12 GONG Chen <chen.sst@gmail.com>
@@ -13,10 +13,10 @@ namespace rime {
 
 class Uniquifier : public Filter {
  public:
-  explicit Uniquifier(Engine *engine);
+  explicit Uniquifier(const Ticket& ticket);
 
-  virtual bool Proceed(CandidateList *recruited,
-                       CandidateList *candidates);
+  virtual void Apply(CandidateList* recruited,
+                     CandidateList* candidates);
 };
 
 }  // namespace rime
