@@ -1,5 +1,5 @@
 //
-// Copyleft 2011 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2011-05-29 GONG Chen <chen.sst@gmail.com>
@@ -7,8 +7,8 @@
 #ifndef RIME_MENU_H_
 #define RIME_MENU_H_
 
+#include <functional>
 #include <vector>
-#include <boost/function.hpp>
 #include <rime/candidate.h>
 #include <rime/common.h>
 
@@ -17,21 +17,20 @@ namespace rime {
 class Translation;
 
 struct Page {
-  int page_size;
-  int page_no;
-  bool is_last_page;
+  int page_size = 0;
+  int page_no = 0;
+  bool is_last_page = false;
   CandidateList candidates;
 };
 
 class Menu {
  public:
-  typedef boost::function<void (CandidateList *recruited,
-                                CandidateList *candidates)>
-  CandidateFilter;
+  using CandidateFilter = std::function<void (CandidateList* recruited,
+                                              CandidateList* candidates)>;
 
-  Menu() {}
-  Menu(const CandidateFilter &filter) : filter_(filter) {}
-  ~Menu() {}
+  Menu() = default;
+  ~Menu() = default;
+  Menu(const CandidateFilter& filter) : filter_(filter) {}
 
   void AddTranslation(shared_ptr<Translation> translation);
   size_t Prepare(size_t candidate_count);
@@ -47,7 +46,7 @@ class Menu {
   }
 
  private:
-  std::vector<shared_ptr<Translation> > translations_;
+  std::vector<shared_ptr<Translation>> translations_;
   CandidateList candidates_;
   CandidateFilter filter_;
 };
