@@ -1,5 +1,5 @@
 //
-// Copyleft 2011 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2012-12-16 GONG Chen <chen.sst@gmail.com>
@@ -8,18 +8,17 @@
 #define RIME_MESSENGER_H_
 
 #include <string>
-#include <boost/signals.hpp>
 #include <rime/common.h>
 
 namespace rime {
 
 class Messenger {
  public:
-  typedef boost::signal<void (const std::string& message_type,
-                              const std::string& message_value)> MessageSink;
-  
+  using MessageSink = signal<void (const std::string& message_type,
+                                   const std::string& message_value)>;
+
   MessageSink& message_sink() { return message_sink_; }
-  
+
  protected:
   MessageSink message_sink_;
 };
