@@ -1,5 +1,5 @@
 //
-// Copyleft 2011 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2011-11-02 GONG Chen <chen.sst@gmail.com>
@@ -15,28 +15,21 @@ namespace rime {
 
 // DbAccessor members
 
-DbAccessor::DbAccessor() {
-}
-
-DbAccessor::DbAccessor(const std::string& prefix) : prefix_(prefix) {
-}
-
-DbAccessor::~DbAccessor() {
-}
-
 bool DbAccessor::MatchesPrefix(const std::string& key) {
   return boost::starts_with(key, prefix_);
 }
 
 // Db members
 
-Db::Db(const std::string& name)
-    : name_(name), loaded_(false), readonly_(false), disabled_(false) {
-  boost::filesystem::path path(Service::instance().deployer().user_data_dir);
-  file_name_ = (path / name).string();
-}
-
-Db::~Db() {
+Db::Db(const std::string& name) : name_(name) {
+  boost::filesystem::path path(name);
+  if (path.has_parent_path()) {
+    file_name_ = name;
+  }
+  else {
+    boost::filesystem::path dir(Service::instance().deployer().user_data_dir);
+    file_name_ = (dir / path).string();
+  }
 }
 
 bool Db::Exists() const {
