@@ -1,5 +1,5 @@
 //
-// Copyleft 2011 RIME Developers
+// Copyleft RIME Developers
 // License: GPLv3
 //
 // 2011-12-10 GONG Chen <chen.sst@gmail.com>
@@ -11,8 +11,8 @@
 #include <boost/uuid/uuid_io.hpp>
 #include <rime_version.h>
 #include <rime/common.h>
-#include <rime/config.h>
 #include <rime/schema.h>
+#include <rime/ticket.h>
 #include <rime/algo/utilities.h>
 #include <rime/dict/dictionary.h>
 #include <rime/dict/dict_compiler.h>
@@ -23,6 +23,8 @@
 #include <windows.h>
 #endif
 
+using namespace std::placeholders;
+
 namespace fs = boost::filesystem;
 
 namespace rime {
@@ -110,7 +112,7 @@ bool InstallationUpdate::Run(Deployer* deployer) {
 bool WorkspaceUpdate::Run(Deployer* deployer) {
   LOG(INFO) << "updating workspace.";
   {
-    scoped_ptr<DeploymentTask> t;
+    unique_ptr<DeploymentTask> t;
     t.reset(new ConfigFileUpdate("default.yaml", "config_version"));
     t->Run(deployer);
     // since brise 0.18
@@ -128,7 +130,7 @@ bool WorkspaceUpdate::Run(Deployer* deployer) {
                << default_config_path.string() << "'.";
     return false;
   }
-  ConfigListPtr schema_list = config.GetList("schema_list");
+  auto schema_list = config.GetList("schema_list");
   if (!schema_list) {
     LOG(WARNING) << "schema list not defined.";
     return false;
@@ -138,14 +140,14 @@ bool WorkspaceUpdate::Run(Deployer* deployer) {
   int success = 0;
   int failure = 0;
   std::map<std::string, std::string> schemas;
-  // fill the prescription
-  for (ConfigList::Iterator it = schema_list->begin();
-       it != schema_list->end(); ++it) {
-    ConfigMapPtr item = As<ConfigMap>(*it);
-    if (!item) continue;
-    ConfigValuePtr schema_property = item->GetValue("schema");
-    if (!schema_property) continue;
-    const std::string &schema_id(schema_property->str());
+  for (auto it = schema_list->begin(); it != schema_list->end(); ++it) {
+    auto item = As<ConfigMap>(*it);
+    if (!item)
+      continue;
+    auto schema_property = item->GetValue("schema");
+    if (!schema_property)
+      continue;
+    const std::string& schema_id(schema_property->str());
     LOG(INFO) << "schema: " << schema_id;
     std::string schema_path;
     if (schemas.find(schema_id) == schemas.end()) {
@@ -160,39 +162,38 @@ bool WorkspaceUpdate::Run(Deployer* deployer) {
       continue;
     }
     // build schema
-    scoped_ptr<DeploymentTask> t(new SchemaUpdate(schema_path));
+    unique_ptr<DeploymentTask> t(new SchemaUpdate(schema_path));
     if (t->Run(deployer))
       ++success;
     else
       ++failure;
   }
   // find dependencies
-  for (std::map<std::string, std::string>::const_iterator s = schemas.begin();
-       s != schemas.end(); ++s) {
+  for (auto s = schemas.cbegin(); s != schemas.cend(); ++s) {
     Config schema_config;
     // user could have customized dependencies in the resulting schema
     std::string user_schema_path = GetSchemaPath(deployer, s->first, false);
     if (!schema_config.LoadFromFile(user_schema_path))
       continue;
-    ConfigListPtr dependencies = schema_config.GetList("schema/dependencies");
+    auto dependencies = schema_config.GetList("schema/dependencies");
     if (!dependencies)
       continue;
-    for (ConfigList::Iterator d = dependencies->begin();
-         d != dependencies->end(); ++d) {
-      ConfigValuePtr dependency = As<ConfigValue>(*d);
-      if (!dependency) continue;
+    for (auto d = dependencies->begin(); d != dependencies->end(); ++d) {
+      auto dependency = As<ConfigValue>(*d);
+      if (!dependency)
+        continue;
       std::string dependency_id(dependency->str());
       if (schemas.find(dependency_id) != schemas.end())  // already built
         continue;
       LOG(INFO) << "new dependency: " << dependency_id;
-      std::string dependency_path(GetSchemaPath(deployer, dependency_id, true));
+      std::string dependency_path = GetSchemaPath(deployer, dependency_id, true);
       schemas[dependency_id] = dependency_path;
       if (dependency_path.empty()) {
         LOG(WARNING) << "missing schema file for dependency '" << dependency_id << "'.";
         continue;
       }
       // build dependency
-      scoped_ptr<DeploymentTask> t(new SchemaUpdate(dependency_path));
+      unique_ptr<DeploymentTask> t(new SchemaUpdate(dependency_path));
       if (t->Run(deployer))
         ++success;
       else
@@ -223,6 +224,15 @@ std::string WorkspaceUpdate::GetSchemaPath(Deployer* deployer,
   return schema_path.string();
 }
 
+SchemaUpdate::SchemaUpdate(TaskInitializer arg) : verbose_(false) {
+  try {
+    schema_file_ = boost::any_cast<std::string>(arg);
+  }
+  catch (const boost::bad_any_cast&) {
+    LOG(ERROR) << "SchemaUpdate: invalid arguments.";
+  }
+}
+
 static std::string find_dict_file(const std::string& dict_file_name,
                                   const fs::path& shared_data_path,
                                   const fs::path& user_data_path) {
@@ -275,15 +285,14 @@ bool SchemaUpdate::Run(Deployer* deployer) {
     return true;
   }
   DictionaryComponent component;
-  scoped_ptr<Dictionary> dict;
-  dict.reset(component.Create(Ticket(&schema, "translator")));
+  unique_ptr<Dictionary> dict(component.Create({&schema, "translator"}));
   if (!dict) {
     LOG(ERROR) << "Error creating dictionary '" << dict_name << "'.";
     return false;
   }
   LOG(INFO) << "preparing dictionary '" << dict_name << "'.";
-  DictFileFinder finder =
-      boost::bind(&find_dict_file, _1, shared_data_path, user_data_path);
+  DictFileFinder finder = std::bind(&find_dict_file,
+                                    _1, shared_data_path, user_data_path);
   DictCompiler dict_compiler(dict.get(), finder);
   if (verbose_) {
     dict_compiler.set_options(DictCompiler::kRebuild | DictCompiler::kDump);
@@ -296,6 +305,17 @@ bool SchemaUpdate::Run(Deployer* deployer) {
   return true;
 }
 
+ConfigFileUpdate::ConfigFileUpdate(TaskInitializer arg) {
+  try {
+    auto p = boost::any_cast< std::pair<std::string, std::string>>(arg);
+    file_name_ = p.first;
+    version_key_ = p.second;
+  }
+  catch (const boost::bad_any_cast&) {
+    LOG(ERROR) << "ConfigFileUpdate: invalid arguments.";
+  }
+}
+
 bool ConfigFileUpdate::Run(Deployer* deployer) {
   fs::path shared_data_path(deployer->shared_data_dir);
   fs::path user_data_path(deployer->user_data_dir);
@@ -315,13 +335,12 @@ bool PrebuildAllSchemas::Run(Deployer* deployer) {
   fs::path user_data_path(deployer->user_data_dir);
   if (!fs::exists(shared_data_path) || !fs::is_directory(shared_data_path))
     return false;
-  fs::directory_iterator iter(shared_data_path);
-  fs::directory_iterator end;
   bool success = true;
-  for (; iter != end; ++iter) {
+  for (fs::directory_iterator iter(shared_data_path), end;
+       iter != end; ++iter) {
     fs::path entry(iter->path());
     if (boost::ends_with(entry.string(), ".schema.yaml")) {
-      scoped_ptr<DeploymentTask> t(new SchemaUpdate(entry.string()));
+      unique_ptr<DeploymentTask> t(new SchemaUpdate(entry.string()));
       if (!t->Run(deployer))
         success = false;
     }
@@ -338,9 +357,8 @@ bool SymlinkingPrebuiltDictionaries::Run(Deployer* deployer) {
     return false;
   bool success = false;
   // test existing link
-  fs::directory_iterator test(user_data_path);
-  fs::directory_iterator end;
-  for (; test != end; ++test) {
+  for (fs::directory_iterator test(user_data_path), end;
+       test != end; ++test) {
     fs::path entry(test->path());
     if (fs::is_symlink(entry) && entry.extension().string() == ".bin") {
       try {
@@ -357,8 +375,8 @@ bool SymlinkingPrebuiltDictionaries::Run(Deployer* deployer) {
     }
   }
   // create link
-  fs::directory_iterator iter(shared_data_path);
-  for (; iter != end; ++iter) {
+  for (fs::directory_iterator iter(shared_data_path), end;
+       iter != end; ++iter) {
     fs::path entry(iter->path());
     fs::path link(user_data_path / entry.filename());
     try {
@@ -382,8 +400,7 @@ bool UserDictUpgration::Run(Deployer* deployer) {
   UserDictList dicts;
   manager.GetUserDictList(&dicts);
   bool ok = true;
-  UserDictList::const_iterator it = dicts.begin();
-  for (; it != dicts.end(); ++it) {
+  for (auto it = dicts.cbegin(); it != dicts.cend(); ++it) {
     if (!manager.UpgradeUserDict(*it))
       ok = false;
   }
@@ -404,37 +421,34 @@ bool BackupConfigFiles::Run(Deployer* deployer) {
   if (!fs::exists(backup_dir)) {
     boost::system::error_code ec;
     if (!fs::create_directories(backup_dir, ec)) {
-      LOG(ERROR) << "error creating directory '" << backup_dir.string() << "'.";
+      LOG(ERROR) << "error creating directory '"
+                 << backup_dir.string() << "'.";
       return false;
     }
   }
   int success = 0, failure = 0, latest = 0, skipped = 0;
-  fs::directory_iterator iter(user_data_path);
-  fs::directory_iterator end;
-  for (; iter != end; ++iter) {
+  for (fs::directory_iterator iter(user_data_path), end;
+       iter != end; ++iter) {
     fs::path entry(iter->path());
-    {
-      bool is_yaml_file = fs::is_regular_file(entry) &&
-          entry.extension().string() == ".yaml";
-      if (!is_yaml_file)
-        continue;
-    }
+    if (!fs::is_regular_file(entry))
+      continue;
+    auto file_extension = entry.extension().string();
+    bool is_yaml_file = file_extension == ".yaml";
+    bool is_text_file = file_extension == ".txt";
+    if (!is_yaml_file && !is_text_file)
+      continue;
     fs::path backup = backup_dir / entry.filename();
-    {
-      bool up_to_date = fs::exists(backup) &&
-          Checksum(backup.string()) == Checksum(entry.string());
-      if (up_to_date) {
-        ++latest;
-        continue;
-      }
+    if (fs::exists(backup) &&
+        Checksum(backup.string()) == Checksum(entry.string())) {
+      ++latest;  // already up-to-date
+      continue;
     }
-    if (!boost::ends_with(entry.string(), ".custom.yaml")) {
+    if (is_yaml_file && !boost::ends_with(entry.string(), ".custom.yaml")) {
       Config config;
       std::string checksum;
-      bool is_customized_copy = config.LoadFromFile(entry.string()) &&
-          config.GetString("customization", &checksum);
-      if (is_customized_copy) {
-        ++skipped;
+      if (config.LoadFromFile(entry.string()) &&
+          config.GetString("customization", &checksum)) {
+        ++skipped;  // customized copy
         continue;
       }
     }
@@ -455,20 +469,19 @@ bool BackupConfigFiles::Run(Deployer* deployer) {
   return !failure;
 }
 
-bool CleanUpTrash::Run(Deployer* deployer) {
+bool CleanupTrash::Run(Deployer* deployer) {
   LOG(INFO) << "clean up trash.";
   fs::path user_data_path(deployer->user_data_dir);
   if (!fs::exists(user_data_path))
     return false;
   fs::path trash = user_data_path / "trash";
   int success = 0, failure = 0;
-  fs::directory_iterator iter(user_data_path);
-  fs::directory_iterator end;
-  for (; iter != end; ++iter) {
+  for (fs::directory_iterator iter(user_data_path), end;
+       iter != end; ++iter) {
     fs::path entry(iter->path());
     if (!fs::is_regular_file(entry))
       continue;
-    std::string filename(entry.filename().string());
+    auto filename = entry.filename().string();
     if (filename == "rime.log" ||
         boost::ends_with(filename, ".userdb.kct.old") ||
         boost::ends_with(filename, ".userdb.kct.snapshot")) {
@@ -505,6 +518,7 @@ bool CleanOldLogFiles::Run(Deployer* deployer) {
   DLOG(INFO) << "today: " << today;
 
   std::vector<std::string> dirs;
+#ifdef RIME_ENABLE_LOGGING
 #ifdef _WIN32
   // work-around: google::GetExistingTempDirectories crashes on windows 7
   char tmp[MAX_PATH];
@@ -513,16 +527,14 @@ bool CleanOldLogFiles::Run(Deployer* deployer) {
 #else
   google::GetExistingTempDirectories(&dirs);
 #endif  // _WIN32
+#endif  // RIME_ENABLE_LOGGING
   DLOG(INFO) << "scanning " << dirs.size() << " temp directory for log files.";
 
   bool success = true;
   int removed = 0;
-  std::vector<std::string>::const_iterator i = dirs.begin();
-  for (; i != dirs.end(); ++i) {
+  for (auto i = dirs.cbegin(); i != dirs.cend(); ++i) {
     DLOG(INFO) << "temp directory: " << *i;
-    fs::directory_iterator j(*i);
-    fs::directory_iterator end;
-    for (; j != end; ++j) {
+    for (fs::directory_iterator j(*i), end; j != end; ++j) {
       fs::path entry(j->path());
       std::string file_name(entry.filename().string());
       try {
